//
//  main.m
//  Personenliste
//
//  Created by Holger Hinzberg on 27.03.11.
//  Copyright 2011 Cocoa-Coding. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Person.h"


int compareFunction(id val1,id val2,void *context)
{
    // Die Zeiger in Personen umwandeln.
    Person *pers1 = val1;
    Person *pers2 = val2;
    
    // Die Vornamen der Personen holen.
    NSString *name1 = pers1.name;
    NSString *name2 = pers2.name;
    
    // Vergleichen und dabei die Groß- und
    // Kleinschreibung ignorieren.
    if([name1 caseInsensitiveCompare:name2] 
       == NSOrderedAscending  )
    {
        return NSOrderedAscending ;
    }
    else if([name1 caseInsensitiveCompare:name2] 
            == NSOrderedDescending)
    {
        return NSOrderedDescending;
    }
    
    return NSOrderedSame;
}


int main (int argc, const char * argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    // 1. Beispiel - NSArray
    /*
    // Drei Personen-Objekte mit 
    // unterschiedlichen Nachnamen erzeugen.
    Person *person1 = [Person new];
    person1.name = @"Meier";
    
    Person *person2 = [Person new];
    person2.name = @"Schmidt";
    
    Person *person3 = [Person new];
    person3.name = @"Schuster";
    
    // Ein NSArray erzeugen und die 
    // drei Personen hinzufügen.
    NSArray *personArray = [[NSArray alloc]
                            initWithObjects:person1,person2,person3, nil];
    
    // Die Anzahl der Objekte im Array ausgeben.
    NSLog(@"Objekte im Array%i", [personArray count]);
    
    // Eine Person an einer angegebenen Position
    // aus dem Array holen.
    Person *indexPerson =[personArray objectAtIndex:1];
    [indexPerson sayYourName];
    */
    
    
    // 2. beispiel - NSMutableArray
    
    // Eine Initialisierung für vier Objekte.
    NSMutableArray *personArray 
    = [[NSMutableArray alloc] initWithCapacity:4];
    
    Person *person1 = [[Person alloc] init];
    person1.name = @"Meier";
    [personArray addObject:person1];
    [person1 release];
    
    person1 = [[Person alloc] init]; 
    person1.name = @"Schmidt";
    [personArray addObject:person1];
    [person1 release];
    
    person1 = [[Person alloc] init];
    person1.name = @"Schuster";
    [personArray addObject:person1];
    [person1 release];
    
    // Anzahl der Objekte ausgeben
    //NSLog(@"Objekte im Array%i", [personArray count]);
    
    // Objekt an Indexposition 1 holen.
    //Person *indexPerson = [personArray objectAtIndex:1];
    
    //[indexPerson sayYourName];
        
    // Schleife für Ausgabe, Version 1
    /*
    for( int i = 0; i < [personArray count]; i++)
    {
        Person *indexPerson 
        =[personArray objectAtIndex:i];
        [indexPerson sayYourName];
    }
    */    
    
    // Schleife für Ausgabe, Version 2
    /*
    for( int i = 0; i < [personArray count]; i++)
    {
        // Verschachtelte Anweisungen
        [[personArray objectAtIndex:i] sayYourName];
    }
    */
    
    // Objekte tauschen mitreplaceObjectAtIndex
    
    // Noch eine Person.
    Person *anotherPerson = [[Person alloc] init];
    anotherPerson.name = @"Koch";
    anotherPerson.firstname = @"Franz";
    
    // Die Person im Array an Position 1 wird
    // durch den neue Objekt ersetzt.
    [personArray replaceObjectAtIndex:1 
                           withObject:anotherPerson];
    
    // Diese Referenz wird nicht mehr benötigt.
    [anotherPerson release];
    
    
    // Objekte tauschen mit replaceObjectsAtIndexes
    
    // Ein zweites Array mit weiteren Personen
    NSMutableArray *morePersonsArray 
    = [[NSMutableArray alloc] initWithCapacity:2];
    
    Person *moreP1 = [Person new];
    moreP1.name = @"Schuhmacher";
    [morePersonsArray addObject:moreP1];
    [moreP1 release];
    
    Person *moreP2 = [Person new];
    moreP2.name = @"Bäcker";
    [morePersonsArray addObject:moreP2];
    [moreP2 release];
    
    // Ein Indexset, welches die 
    // Positionen 0 und 1 beschreibt.
    NSIndexSet *rangeIndex 
    = [NSIndexSet indexSetWithIndexesInRange:
       NSMakeRange(0, 2)];
    
    // Die Positionen im Indexset durch die
    // Personen im neuen Array ersetzten.
    [personArray replaceObjectsAtIndexes:
     rangeIndex withObjects:morePersonsArray];
    

    // Ein Person einfügen
    
    // Noch eine neue Person
    Person *insertPerson = [Person new];
    insertPerson.name = @"Gärtner";
    
    // Die Person an Position 1 dem Array hinzufügen
    [personArray insertObject:insertPerson atIndex:1];
    [insertPerson release];
    
    // Objekte entfernen
    
    // Eine bestimmte Person entfernen.
    // [personArray removeObject:insertPerson];
    
    // Person an Position 0 entfernen.
    // [personArray removeObjectAtIndex:0];
    
    
    // Das Array sortieren mit einer Funktion
    // [personArray sortUsingFunction: compareFunction context:NULL];
    
    // Das Array sortieren mit einer Methode
    [personArray sortUsingSelector:@selector(comparePerson:)];
    
    
    // Schleife mit Fast Enumeration
    for(Person *aPerson in personArray)
    {
        [aPerson sayYourName];
    }
    
    [pool drain];
    return 0;
}

